
// ParserGen.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Jan 2018

/* Top-level of the compiler which links together
   parsing (in Parser) and code generation (in CodeGen).
*/


public class Cabbage
{

  public static void main(String args[]) throws java.io.IOException
  {
   if (args.length == 0) {
      System.out.println("Usage: java Cabbage [-t] <CB fnm>");
      return;
    }

    boolean isTracing = false;
    String fnm = null;
    if (args.length == 1)
      fnm = args[0];
    else {
      isTracing = (args[0].equals("-t"));
      fnm = args[1];
    }

    // parse program into intermediate code
    CodeGen cg = new CodeGen();
    Parser parser = new Parser(fnm, cg);
    parser.program();

    cg.derefForwards( parser.getFuncLabels() );

    // add AE instructions to intermediate code 
    cg.defineVars();
    cg.convertJMPs();

    cg.setVarAddrs();
    if (isTracing)
      cg.showVars();

    cg.genCards();
    if (isTracing)
      cg.showCode();

    // save the AE instructions to a file
    String outFnm = fnm.substring(0, 
                        fnm.lastIndexOf(".")) + ".ae";
    cg.saveCards(outFnm);
  }  // end of main()


}  // end of Cabbage class
