
// CodeLine.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Feb 2018

/* A line of intermediate code. The CodeGen class
   generates a list of these lines which will be
   converted into AE instructions later.
*/


import java.io.*;


public class CodeLine
{
  private CodeOps op;   
      // this is the only field guaranteed to have a value;
      // the other fields depend on the type of op

  private String varName;

  private long value;    
      /* this is used for various forms of numerical data,
         including the value of a variable, and a shift amount
      */

  private int jumpLabel;
  private String arrayName;

  // private int address;

  private String card;   
        // the AE instruction corresponding to this code



  public CodeLine(CodeOps op, String nm)
  { this.op = op;
    varName = nm;
    value = -1;
    jumpLabel = -1;
    arrayName = null;
    // address = -1;
    card = null;
  }

  public CodeOps getOp()
  {  return op;  }

  public void setOp(CodeOps op)
  {  this.op = op;  }


  public String getVar()
  {  return varName;  }

  public void setVar(String nm)
  {  varName = nm; }


  public long getValue()
  {  return value; }

  public void setValue(long n)
  {  value = n; }


  public int getLabel()
  {  return jumpLabel;  }

  public void setLabel(int l)
  {  jumpLabel = l; }



  public String getArrayName()
  {  return arrayName;  }

  public void setArrayName(String nm)
  {  arrayName = nm; }


  public String getCard()
  { return card; }

  public void setCard(String s)
  {  card = s; }

/*
  public int getAddress()
  {  return address; }

  public void setAddress(int addr)
  {  address = addr; }
*/


  public String toString()
  {  return getDetails(true);  }


  public String getDetails(boolean withCard)
  {  
    StringBuilder sb = new StringBuilder("  ");
    if (jumpLabel != -1)
      sb.append(jumpLabel + ": ");

    sb.append(op + " ");

    if (varName != null) {
      sb.append(varName);
      if (arrayName != null)
        sb.append(" " + arrayName + " ");
      else
        sb.append(" ");
    }
/*
    if (address != -1) {
      sb.append(address);
      if (value != -1)
        sb.append(" " + value + " ");
      else
        sb.append(" ");
    }
*/
    if (withCard && (card != null))
      sb.append("\t\t\t\"" + card + "\"");
    return sb.toString();
  }  // end of getDetails()



}  // end of CodeLine class
