
// CodeOps.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Feb 2018

/* These are the operations used by the intermediate code
   language generated by CodeGen. They will be translated 
   into AE instructions in another pass over the code.
*/

public enum CodeOps
{
  NOP("nop"), 

  NUM("num"), 
  NUMA("numa"),       // will later hold the start address of an array,
                      // when it will be changed to a NUM instruction

  LOAD("load"), 
  LOADI("loadi"),     // "i" == intermediate
  LOADPC("loadpc"),   // load the PC into the mill

  STORE("store"), 
  STOREI("storei"), 
  STOREP("storep"),    // "p" == primed
  STOREIP("storeip"), 

  ADD("add"), 
  SUB("sub"), 
  MULT("mult"), 
  DIV("div"),

  JMPZ("jmpz"),        // JMP if test result is 0
  JMP("jmp"),          // unconditional jump
  JMPF("jmpf"),        // jump to a function that is not yet defined;
                       // (will be replaced by JMP at code-gen time)
  JMPV("jmpv"),        // jump by an offset distance stored in the mill

  PRINT("print"), 
  PUTCHAR("putchar"), 

  READ("read"), 

  RIGHT(">"), 
  LEFT("<"), 

  EXT("ext"), 

  // room for growth...

  HALT("halt");


  private String text;


  CodeOps(String text) 
  {  this.text = text; }


  public String getText() 
  {  return this.text; }


  public static CodeOps fromString(String text) 
  {
    for (CodeOps b : CodeOps.values()) {
      if (b.text.equalsIgnoreCase(text))
        return b;
    }
    return NOP;
  }  // end of fromString()


}  // end of CodeOps class
