
/* Lexer.jflex  */
/* Andrew Davison, ad@fivedots.coe.psu.ac.th, Jan 2018  */


%%
%{
  private void echo() 
  { System.out.print(yytext()); }

  public int position() 
  { return yycolumn; }

  public void error(int pos, String msg)
  {
    System.out.println();
    for (int i = 0; i < pos; i++) 
      System.out.print(" ");
    System.out.println("^");
    System.out.println("***** Error: " + msg + " *****");
    System.exit(0);
  }  // end of error()

%}

%class    Lexer
%function nextToken
%type	  Token

%unicode
%line
%column
/* %ignorecase */

%eofval{
  { return new Token( Toks.EOF); }
%eofval}

Identifier = [:letter:]([:letter:] | [:digit:])*
Integer    = [:digit:]+
Number     = {Integer} ( \. {Integer} )?


%%
[ \t\b\f\r\n]+	{ echo(); /* eat whitespace */ }

"//"[^\n]*      { echo();  /* one-line comment */ }


";"		{ echo(); return new Token( Toks.SEMICOLON); }
/* "."		{ echo(); return new Token( Toks.PERIOD); } */
","		{ echo(); return new Token( Toks.COMMA); }

"<"		{ echo(); return new Token( Toks.LT); }
"<="		{ echo(); return new Token( Toks.LE); }
">"		{ echo(); return new Token( Toks.GT); }
">="		{ echo(); return new Token( Toks.GE); }
"=="		{ echo(); return new Token( Toks.EQ); }
"<>"		{ echo(); return new Token( Toks.NE); }

"("		{ echo(); return new Token( Toks.LPAREN); }
")"		{ echo(); return new Token( Toks.RPAREN); }
"&&"		{ echo(); return new Token( Toks.AND); }
"||"		{ echo(); return new Token( Toks.OR); }

"+"		{ echo(); return new Token( Toks.PLUS); }
"-"		{ echo(); return new Token( Toks.MINUS); }
"*"		{ echo(); return new Token( Toks.TIMES); }
"/"		{ echo(); return new Token( Toks.DIV); }
"="		{ echo(); return new Token( Toks.ASSIGN); }
"{"		{ echo(); return new Token( Toks.BEGIN); }
"}"		{ echo(); return new Token( Toks.END); }
"["		{ echo(); return new Token( Toks.LBRACE); }
"]"		{ echo(); return new Token( Toks.RBRACE); }

if		{ echo(); return new Token( Toks.IF); }
else		{ echo(); return new Token( Toks.ELSE); }
while		{ echo(); return new Token( Toks.WHILE); }
print		{ echo(); return new Token( Toks.PRINT); }
read		{ echo(); return new Token( Toks.READ); }
dim 		{ echo(); return new Token( Toks.DIM); }
numdps 		{ echo(); return new Token( Toks.NUMDPS); }

func 		{ echo(); return new Token( Toks.FUNC); }
call 		{ echo(); return new Token( Toks.CALL); }
return 		{ echo(); return new Token( Toks.RETURN); }
ext 		{ echo(); return new Token( Toks.EXT); }

{Number}    { echo(); return new Token( Toks.NUMBER, yytext()); }

{Identifier}	{ echo(); return new Token( Toks.ID, yytext()); }

/* matches string-literal on a single line*/
\"([^\"\\]|\\.)*\"  { echo(); return new Token( Toks.STRING, yytext()); }

.		{ echo(); error(yychar, "Illegal character"); }
