
// CardReader.java

/* Fourmilab's Analytical Engine, by John Walker
   https://www.fourmilab.ch/babbage/contents.html

   Cut down functionality by:
   Andrew Davison, ad@fivedots.coe.psu.ac.th, Feb 2018

   This class represents the chain of cards used to input
   data and instructions to the engine. Although it is
   implemented as an in-memory ArrayList, it is meant 
   to represent an external sequence of cards which
   can be moved through in both a forward and backward
   direction. 

   This movement capability is crucial to implementing
   jumps at the software level.
*/

import java.io.*;
import java.util.*;
import java.math.*;


public class CardReader
{
  private ArrayList<String> cards;
  private int cardIdx = 0;


  public CardReader(String fnm, boolean isTracing)
  {
    cards = new ArrayList<String>();
    readCards(fnm, isTracing);
  }


  private void readCards(String fnm, boolean isTracing)
  // ad: replaced more complex code with BufferedReader
  {
		try (BufferedReader br = new BufferedReader(
                                 new FileReader(fnm))) {
			String currLine;
			while ((currLine = br.readLine()) != null) {
				// System.out.println(currLine);
        cards.add(currLine);
			}
      if (isTracing)
        list();
		} 
    catch (IOException e) {
			System.out.println("Could not read file: " + fnm);
      System.exit(1);
		}
  } // end of readCards()



  public String getCard()
  {
    try {
      return cards.get(cardIdx++);
    }
    catch (Exception e) 
    { System.out.println("No card found ");
      return null; 
    }
  }


  public String getCard(int idx)
  // ad: used by the new "X" card
  {
    if ((idx < 0) || (idx >= cards.size())) {
      System.out.println("Card index out of range");
      return null;
    }
    else
      return cards.get(idx);
  }


  public int getIndex()
  // added by ad, so that program counter is accessible;
  /* in "raw" Cabbage, the card index is not accessible
     at the user level, only jumpForward() and jumpBack()
  */
  {  return cardIdx;  }


  public boolean jumpForward(int n)
  {
    if ((cardIdx + n) >= cards.size())
      return false;
    cardIdx += n;
    return true;
  }


  public boolean jumpBack(int n)
  {
    if ((cardIdx - n) < 0)
      return false;
    cardIdx -= n;
    return true;
  }


  public void list()
  { 
    System.out.println("\nCards:");
    System.out.println("----------");
    for (String c : cards)
      System.out.println(c);
    System.out.println("----------");
  }

}  // end of CardReader class
