
// Utils.java
// Andrew Davison, ad@fivedots.coe.psu.ac.th, Feb 2018

/* Contains public static Java methods that can be called from
   the engine.

   The functions must take only double arguments, and return
   a double. A function name must be made from lowercase letters,
   digits, or "_".
*/

import java.time.*;


public class Utils
{
  private static final int MAX = 10000;
  private static int last = (int) (System.currentTimeMillis() % MAX);
  


  public static double add(double x, double y)
  {  return x+y;  }

  public static double max(double x, double y)
  {  return (x > y) ? x : y;  }

  public static double log(double x)
  {  return Math.log(x);  }

  public static double random()
  {  return Math.random();  }

  public static double time()
  {  return (double) LocalTime.now().toSecondOfDay();  }


  public static int randomInt()
  { last = (last*32719 + 3) % 32749;
    return last % MAX;
  }

  public static double randomDouble()
  { return ((double)randomInt()) / MAX;  }



  public static double myLog(double x)
  {
    if (x <= 0) {
      System.out.println("myLog() value must be positive");
      return 0;
    }

    double term = (x-1)/(x+1);
    double multiple = term*term;
    double sum = term;
    int i = 1;
    while (term > 0.000000001) {
      i = i+2;
      term = (term * multiple);
      sum += term/i;
    }
    return sum*2;
  }  // end of myLog()


  // ------------------ test rig ---------------

  public static void main(String args[])
  {
    if (args.length != 1) {
      System.out.println("Usage: java Utils <double>");
      return;
    }

    double val = 1.0;
    try {
      val = Double.parseDouble(args[0]);
    }
    catch(NumberFormatException e)
    {  System.out.println("Wrong format; using 1.0");  }


    // System.out.println("myLog(" + val + "): " + myLog(val));
    System.out.println("rand 1: " + randomInt());
    System.out.println("rand 2: " + randomDouble());
  }  // end of main()



}  // end of Utils class